#include "camera.hpp"
#include "input.hpp"

Camera::Camera()
: position (Vector(0, 0, 0))
, target (Vector(0, 0, 0))
{
}

Camera::Camera(Vector const & position)
: position (position)
, target (Vector(0, 0, 0))
{
}

Camera::~Camera()
{
}

void Camera::SetTarget(Vector const & target)
{
    this->target = target;

    /**
     * Update view projection matrix.
     */

    vp.LoadIdentity();

    /**
     * View.
     */

    Matrix view;

    {
        Vector pos = Vector(-target.x - position.x,
                            -target.y - position.y,
                            -target.z - position.z);

        view.LoadIdentity();
        view.Translate(pos);
        view.Mult(Quaternion(Vector(1, 0, 0), 45 * (pi / 180.f)).ToMatrix());

        /**

        Can't get the lookAt matrix to work.

        view.LookAt(Vector(position.x, position.y, -position.z),
                    Vector(0, 0, 0),
                    Vector(0, 1, 0));
        **/
    }

    /**
     * Projection.
     */

    {
        float const fNear = 0.01f;
        float const fFar = 5.f;
        float const aspect = 640.f / 480.f;

        Matrix scale(640.f, 0, 0, 0,
                     0, -(480.f * aspect), 0, 0,
                     0, 0, 1, 0,
                     0, 0, 0, 1);

        Matrix perspectiveProjection;
        perspectiveProjection.LoadIdentity();
        perspectiveProjection.Perspective(1.25f, aspect, -fNear, -fFar, true);

        Matrix toScreenCenter;
        toScreenCenter.LoadIdentity();
        toScreenCenter.Translate(Vector(0.5f, -0.5f / aspect, 0));

        vp.LoadIdentity();
        vp.Mult(view);
        vp.Mult(perspectiveProjection);
        vp.Mult(toScreenCenter);
        vp.Mult(scale);
    }
}

void Camera::Process(float deltaTime, class Input * pInput)
{
}

Matrix const & Camera::GetViewProjection()
{
    return vp;
}