#include <iostream>
#include <cmath>
#include <cstdlib>
using std::cout;
using std::endl;

#include "enemy.hpp"
#include "player.hpp"

float const movementSpeed = 1;

Enemy::Enemy()
: position (Vector(0, 0, 0))
, direction (Vector(0, 0, 0))
, orientation (0)
, target (NULL)
, behaviour (Wander)
, timer (0)
{
}

Enemy::~Enemy()
{
}

void Enemy::SetStaticMesh(StaticMesh const & staticMesh)
{
    this->staticMesh = staticMesh;
}

void Enemy::SetViewProjection(Matrix const & vp)
{
    this->vp = vp;   
}

void Enemy::Process(float deltaTime, class Input * pInput)
{
    switch(behaviour)
    {
        case Wander:
        {
            timer -= deltaTime;

            if(timer <= 0)
            {
                float dx = rand() / (float) RAND_MAX - 0.5f;
                float dy = rand() / (float) RAND_MAX - 0.5f;

                direction = Vector::Normalize(Vector(dx, 0, dy));
                direction.Scale((rand() / (float) RAND_MAX) * 2);

                timer = (((rand() / (float) RAND_MAX) * 3) + 2) * 1000;
            }
        }
            break;

        case Pursue:
        {

        }
            break;

        case Attack:
        {

        }
            break;

        case Flee:
        {

        }
            break;
    }

    /**
     * Update position.
     */

    Vector prev = position;

    position.x += direction.x * 0.0005f * movementSpeed * deltaTime;
    position.z -= direction.z * 0.0005f * movementSpeed * deltaTime;

    Vector deltaPosition = Vector::Sub(position, prev);

    if(Vector::Length(deltaPosition) != 0)
    {
        deltaPosition = Vector::Normalize(deltaPosition);

        /**
        * Rotate Enemy to face new direction.
        */
        
        orientation = atan2(deltaPosition.x, -deltaPosition.z);
    }
}

void Enemy::Draw()
{
    /**
     * Apply transformation matrix.
     */

    Matrix model;
    model.LoadIdentity();
    model.Mult(Quaternion(Vector(0, 1, 0), orientation).ToMatrix());
    model.Translate(position);
    model.Mult(vp);
    
    staticMesh.SetTransform(model);
    staticMesh.Draw();
}

Vector const & Enemy::GetPosition() const
{
    return position;
}

void Enemy::SetPosition(Vector const & position)
{
    this->position = position;
}

void Enemy::SetTarget(Player * player)
{
    this->target = player;
}