#include <iostream>
#include <cmath>
using std::cout;
using std::endl;

#include "player.hpp"
#include "input.hpp"

float const movementSpeed = 1;

Player::Player()
: position (Vector(0, 0, 0))
, orientation (0)
{
}

Player::~Player()
{
}

void Player::SetStaticMesh(StaticMesh const & staticMesh)
{
    this->staticMesh = staticMesh;
}

void Player::SetViewProjection(Matrix const & vp)
{
    this->vp = vp;   
}

void Player::Process(float deltaTime, class Input * pInput)
{
    cont_state_t * st = pInput->pControllerState;

    if(st != NULL)
    {
        Vector prev = position;

        position.x += st->joyx * 0.00002f * movementSpeed * deltaTime;
        position.z -= st->joyy * 0.00002f * movementSpeed * deltaTime;

        Vector deltaPosition = Vector::Sub(position, prev);

        if(Vector::Length(deltaPosition) != 0)
        {
            deltaPosition = Vector::Normalize(deltaPosition);

            /**
            * Rotate player to face new direction.
            */
            
            orientation = atan2(deltaPosition.x, -deltaPosition.z);
        }
    }
}

void Player::Draw()
{
    /**
     * Apply transformation matrix.
     */

    Matrix model;
    model.LoadIdentity();
    model.Mult(Quaternion(Vector(0, 1, 0), orientation).ToMatrix());
    model.Translate(position);
    model.Mult(vp);

    staticMesh.SetTransform(model);
    staticMesh.Draw();
}

Vector const & Player::GetPosition() const
{
    return position;
}

void Player::SetPosition(Vector const & position)
{
    this->position = position;
}