#include <iostream>
using std::cout;
using std::endl;

#include "static_mesh.hpp"
#include "input.hpp"
#include "render.hpp"
#include "texture.hpp"

StaticMesh::StaticMesh()
: texture (NULL)
, mesh (NULL)
, textureWidth (0)
, textureHeight (0)
{
}

StaticMesh::~StaticMesh()
{
}

StaticMesh StaticMesh::LoadFromFile(std::string const & sMeshFile,
                                    std::string const & sTextureFile,
                                    uint32_t width,
                                    uint32_t height)
{
    tSharedMesh mesh = std::make_shared<Mesh>();

    mesh->LoadFromFile(sMeshFile);
    
    pvr_ptr_t texture;
    load_texture(&texture, width * height * 2, sTextureFile.c_str());

    StaticMesh staticMesh;
    staticMesh.SetMesh(mesh);
    staticMesh.SetTexture(texture, width, height);

    return staticMesh;
}

void StaticMesh::SetMesh(tSharedMesh mesh)
{
    this->mesh = mesh;
}

void StaticMesh::SetTexture(void * texture, uint32_t width, uint32_t height)
{
    this->texture = texture;
    this->textureWidth = width;
    this->textureHeight = height;
}

void StaticMesh::SetTransform(Matrix const & transform)
{
    this->transform = transform;
}

void StaticMesh::Process(float deltaTime, class Input * pInput)
{
}

void StaticMesh::Draw()
{
    if(!mesh || texture == NULL)
    {
        return;
    }

    /**
     * Transform triangles.
     */

    tTris tris = mesh->GetTris();

    for(size_t i = 0; i < tris.size(); ++i)
    {
        Tri & t = tris[i];

        Vector v[3] = {
            Vector(t.v0.x, t.v0.y, t.v0.z),
            Vector(t.v1.x, t.v1.y, t.v1.z),
            Vector(t.v2.x, t.v2.y, t.v2.z)
        };

        bool bSkip = false;

        for(size_t j = 0; j < 3 && bSkip == false; ++j)
        {
            Transform(v[j], transform);

            if(v[j].w <= 0)
            {
                bSkip = true;
            }
        }

        if(bSkip)
        {
            continue;
        }

        for(size_t j = 0; j < 3; ++j)
        {
            v[j].Dehomogenize();

            t.v0 = Vertex(v[0].x, v[0].y, v[0].z, t.v0.u, t.v0.v, t.v0.r, t.v0.g, t.v0.b);
            t.v1 = Vertex(v[1].x, v[1].y, v[1].z, t.v1.u, t.v1.v, t.v1.r, t.v1.g, t.v1.b);
            t.v2 = Vertex(v[2].x, v[2].y, v[2].z, t.v2.u, t.v2.v, t.v2.r, t.v2.g, t.v2.b);
        }
    }

    /**
     * Render triangles.
     */

    render::draw_textured_tris(tris, static_cast<pvr_ptr_t>(texture), textureWidth, textureHeight);
}