#include <iostream>
using std::cout;
using std::endl;

#include "skeletal_mesh_test.hpp"
#include "input.hpp"
#include "render.hpp"
#include "texture.hpp"

SkeletalMeshTest::SkeletalMeshTest()
: firstAngle (0)
, secondAngle (0)
, aPress (0)
, bPress (0)
, quadRenderMode (all)
, skinningMode (eSkinned)
{
    /**
     * Mesh test.
     */

    mesh.LoadFromFile("rd/test.ross_mesh");

    /**
     * Texture.
     */

    pvr_ptr_t thumbs_up;

    float const textureWidth = 256;
    float const textureHeight = 256;

    {
        load_texture(&thumbs_up, textureWidth * textureHeight * 2, "rd/floor.png");
    }

    this->thumbs_up = thumbs_up;

    /**
     * Geometry.
     */

    {
        typedef tWeight W;

        Quad bottom(Vertex(-0.5f, 1.f, 1, 0, 0, 0, 0, 1, Weights(W(1, 0.5f), W(2, 0.5f), W(0, 0))),
                    Vertex(0.5f, 1.f, 1, 1, 0, 0, 0, 1, Weights(W(1, 0.5f), W(2, 0.5f), W(0, 0))),
                    Vertex(-0.5f, 0.f, 1, 0, 1, 0, 0, 1, Weights(W(1, 1), W(0, 0), W(0, 0))),
                    Vertex(0.5f, 0.f, 1, 1, 1, 0, 0, 1, Weights(W(1, 1), W(0, 0), W(0, 0))));

        quads.push_back(bottom);

        Quad middle(Vertex(-0.5f, 2.f, 1, 0, 0, 0, 0, 1, Weights(W(2, 0.5f), W(3, 0.5f), W(0, 0))),
                    Vertex(0.5f, 2.f, 1, 1, 0, 0, 0, 1, Weights(W(2, 0.5f), W(3, 0.5f), W(0, 0))),
                    Vertex(-0.5f, 1.f, 1, 0, 1, 0, 0, 1, Weights(W(1, 0.5f), W(2, 0.5f), W(0, 0))),
                    Vertex(0.5f, 1.f, 1, 1, 1, 0, 0, 1, Weights(W(1, 0.5f), W(2, 0.5f), W(0, 0))));

        quads.push_back(middle);

        Quad top(Vertex(1.f, 2.5f, 1, 0, 0, 0, 0, 1, Weights(W(3, 1), W(0, 0), W(0, 0))),
                 Vertex(1.f, 1.5f, 1, 1, 0, 0, 0, 1, Weights(W(3, 1), W(0, 0), W(0, 0))),
                 Vertex(0.f, 2.5f, 1, 0, 1, 0, 0, 1, Weights(W(2, 0.5), W(3, 0.5f), W(0, 0))),
                 Vertex(0.f, 1.5f, 1, 1, 1, 0, 0, 1, Weights(W(2, 0.5), W(3, 0.5f), W(0, 0))));

        quads.push_back(top);
    }

    /**
     * Skeleton.
     */

    Quaternion r0(Vector(0, 0, 0), 0.f);
    Quaternion r1(Vector(0, 0, 0), 0.f);
    Quaternion r2(Vector(0, 0, 1), 90.f * (pi / 180.f));

    Joint j0(1, Vector(0, 0, 0), r0, 1);
    Joint j1(2, Vector(0, 1, 0), r1, 1);
    Joint j2(3, Vector(0, 1, 0), r2, 1);

    pBindPoseRoot = new Joint(j0);
    new Joint(j1, pBindPoseRoot);
    new Joint(j2, pBindPoseRoot->GetChild(0));
    CalculateInverseBindPose(pBindPoseRoot);

    pDynamicPoseRoot = new Joint(j0);
    new Joint(j1, pDynamicPoseRoot);
    new Joint(j2, pDynamicPoseRoot->GetChild(0));
}

SkeletalMeshTest::~SkeletalMeshTest()
{
    delete pBindPoseRoot;
    delete pDynamicPoseRoot;
}

Vector cameraPos(0, 0, 0, 1);

void SkeletalMeshTest::Process(float deltaTime, Input * pInput)
{
    cont_state_t * st = pInput->pControllerState;

    if(st != NULL)
    {
        if(st->buttons & CONT_B)
        {
            if(bPress == false)
            {
                quadRenderMode = QuadRenderMode(((int)quadRenderMode + 1) % maxQuadRenderMode);
                bPress = true;
            }
        }
        else
        {
            bPress = false;
        }

        if(st->buttons & CONT_A)
        {
            if(aPress == false)
            {
                skinningMode = SkinningMode(((int)skinningMode + 1) % maxSkinningMode);
                aPress = true;
            }
        }
        else
        {
            aPress = false;
        }

        cameraPos.x -= st->joyx * 0.00005f * deltaTime;
        cameraPos.y += st->ltrig * 0.00005f * deltaTime;
        cameraPos.y -= st->rtrig * 0.00005f * deltaTime;

        //firstAngle += st->joyx * 0.002f * deltaTime;
        //secondAngle -= st->ltrig * 0.002f * deltaTime;
        //secondAngle += st->rtrig * 0.002f * deltaTime;

        cameraPos.z += st->joyy * 0.00005f * deltaTime;
    }

    Quaternion q = Quaternion(Vector(0, 0, 1), firstAngle * (pi / 180.f));
    Quaternion q2 = Quaternion(Vector(0, 0, 1), secondAngle * 1.2f * (pi / 180.f));

    pDynamicPoseRoot->rot = q;
    pDynamicPoseRoot->GetChild(0)->rot = q2;

    CalculateForwardPose(pDynamicPoseRoot);
}

float const fNear = 0.01f;
float const fFar = 5.f;

void SkeletalMeshTest::Draw()
{
    tQuads renderQuads;

    /**
     * Transform from engine space to screen space.
     */

     Matrix view;

    {
        /**
         * View matrix.
         */

        view.LoadIdentity();
        view.Translate(cameraPos);

        /**
        view.LookAt(Vector(cameraPos.x, cameraPos.y, -cameraPos.z),
                    Vector(0, 0, 0),
                    Vector(0, 1, 0));
                    **/
    }

    Matrix trans;

    {
        /**
         * Projection matrix.
         */

        float aspect = 640.f / 480.f;

        Matrix scale(640.f, 0, 0, 0,
                     0, -(480.f * aspect), 0, 0,
                     0, 0, 1, 0,
                     0, 0, 0, 1);

        Matrix perspectiveProjection;
        perspectiveProjection.LoadIdentity();
        perspectiveProjection.Perspective(1.5f, aspect, -fNear, -fFar, true);

        Matrix toScreenCenter;
        toScreenCenter.LoadIdentity();
        toScreenCenter.Translate(Vector(0.5f, -0.5f / aspect, 0));

        trans.LoadIdentity();
        trans.Mult(view);
        trans.Mult(perspectiveProjection);
        trans.Mult(toScreenCenter);
        trans.Mult(scale);
    }

    /**
     * Skin quads.
     */

    for(size_t i = 0; i < quads.size(); ++i)
    {
        Quad q = quads[i];

        Vector v0(q.v0.x, q.v0.y, q.v0.z),
               v1(q.v1.x, q.v1.y, q.v1.z),
               v2(q.v2.x, q.v2.y, q.v2.z),
               v3(q.v3.x, q.v3.y, q.v3.z);

        Matrix m;
        m.LoadIdentity();

        Weights weightsEachVertex[4] = {
            q.v0.weights,
            q.v1.weights,
            q.v2.weights,
            q.v3.weights
        };

        Vector verts[4] = {
            v0, v1, v2, v3
        };

        if(skinningMode != eNoSkinning)
        {
            for(size_t j = 0; j < 4; ++j)
            {
                Vector & v = verts[j];

                Vector weightedVerts[3] = {
                    v, v, v
                };

                Weights & _ws = weightsEachVertex[j];

                tWeight weights[3] = {
                    _ws.w0, _ws.w1, _ws.w2
                };

                bool appliedWeight[3] = {
                    false, false, false
                };

                size_t appliedWeightCount = 0;

                for(size_t k = 0; k < 3; ++k)
                {
                    tWeight & w = weights[k];
                    Vector & wv = weightedVerts[k];

                    Joint * pBindPoseJoint = GetJointByID(pBindPoseRoot, w.first);
                    Joint * pDynamicPoseJoint = GetJointByID(pDynamicPoseRoot, w.first);

                    if(pBindPoseJoint != NULL && pDynamicPoseJoint != NULL)
                    {
                        switch(skinningMode)
                        {
                            case eInverseBindPose:
                                Transform(wv, pBindPoseJoint->pose);
                                break;

                            case eSkinned:
                                Transform(wv, pBindPoseJoint->pose);
                                Transform(wv, pDynamicPoseJoint->pose);
                                break;

                            default:
                                break;
                        }

                        appliedWeight[k] = true;
                        ++appliedWeightCount;                      
                    }
                }

                if(appliedWeightCount > 0)
                {
                    Vector runningTotal(0, 0, 0);

                    for(size_t k = 0; k < 3; ++k)
                    {
                        if(appliedWeight[k])
                        {
                            weightedVerts[k].Scale(weights[k].second);
                            runningTotal.Add(weightedVerts[k]);
                        }
                    }

                    v = runningTotal;
                }
            }
        }

        m.Mult(trans);

        for(size_t j = 0; j < 4; ++j)
        {
            Transform(verts[j], m);
        }

        v0 = verts[0];
        v1 = verts[1];
        v2 = verts[2];
        v3 = verts[3];

        v0.Dehomogenize();
        v1.Dehomogenize();
        v2.Dehomogenize();
        v3.Dehomogenize();
        
        q.v0 = Vertex(v0.x, v0.y, v0.z, q.v0.u, q.v0.v);
        q.v1 = Vertex(v1.x, v1.y, v1.z, q.v1.u, q.v1.v);
        q.v2 = Vertex(v2.x, v2.y, v2.z, q.v2.u, q.v2.v);
        q.v3 = Vertex(v3.x, v3.y, v3.z, q.v3.u, q.v3.v);

        bool bShouldDraw = false;

        switch(quadRenderMode)
        {
            case first:
                bShouldDraw = i == 0;
                break;

            case second:
                bShouldDraw = i == 1;
                break;

            case third:
                bShouldDraw = i == 2;
                break;

            case all:
                bShouldDraw = true;
                break;

            default:
                break;
        }

        if(bShouldDraw)
        {
            renderQuads.push_back(q);
        }
    }

    Joint * pDynamicPoseJoint = pDynamicPoseRoot;

    Matrix cumulativeTransform;
    cumulativeTransform.LoadIdentity();

    /**
     * Draw skeleton.
     */

    while(pDynamicPoseJoint != NULL)
    {
        Quad q(Vertex(0, 1, 1, 0, 0),
                Vertex(0.25f, 0.33f, 1, 1, 0),
                Vertex(-0.25f, 0.33f, 1, 0, 1),
                Vertex(0, 0, 1, 1, 1));

        Vector v0(q.v0.x, q.v0.y, q.v0.z),
                v1(q.v1.x, q.v1.y, q.v1.z),
                v2(q.v2.x, q.v2.y, q.v2.z),
                v3(q.v3.x, q.v3.y, q.v3.z);

        float s = pDynamicPoseJoint->scale;

        Matrix scale(s, 0, 0, 0,
                     0, s, 0, 0,
                     0, 0, s, 0,
                     0, 0, 0, 1);

        Matrix m;
        m.LoadIdentity();
        m.Mult(scale);
        m.Mult(pDynamicPoseJoint->rot.ToMatrix());
        m.Translate(pDynamicPoseJoint->loc);
        m.Mult(cumulativeTransform);

        cumulativeTransform = m;

        m.Mult(trans);

        Transform(v0, m);
        Transform(v1, m);
        Transform(v2, m);
        Transform(v3, m);

        /**
         * Convert from homogeneous coordinates.
         */

        v0.Dehomogenize();
        v1.Dehomogenize();
        v2.Dehomogenize();
        v3.Dehomogenize();
        
        q.v0 = Vertex(v0.x, v0.y, v0.z, q.v0.u, q.v0.v);
        q.v1 = Vertex(v1.x, v1.y, v1.z, q.v1.u, q.v1.v);
        q.v2 = Vertex(v2.x, v2.y, v2.z, q.v2.u, q.v2.v);
        q.v3 = Vertex(v3.x, v3.y, v3.z, q.v3.u, q.v3.v);

        renderQuads.push_back(q);
        pDynamicPoseJoint = pDynamicPoseJoint->GetChild(0);
    }

    /**
     * Transform mesh triangles into screen space.
     */

    tTris renderTris;
    tTris & tris = mesh.GetTris();

    for(size_t i = 0; i < tris.size(); ++i)
    {
        Tri t = tris[i];

        Vector v[3] = {
            Vector(t.v0.x, t.v0.y, t.v0.z),
            Vector(t.v1.x, t.v1.y, t.v1.z),
            Vector(t.v2.x, t.v2.y, t.v2.z)
        };

        bool bSkip = false;

        for(size_t j = 0; j < 3 && bSkip == false; ++j)
        {
            Transform(v[j], trans);

            if(v[j].w <= 0)
            {
                bSkip = true;
            }
        }

        if(bSkip)
        {
            continue;
        }

        for(size_t j = 0; j < 3; ++j)
        {
            v[j].Dehomogenize();

            t.v0 = Vertex(v[0].x, v[0].y, v[0].z, t.v0.u, t.v0.v);
            t.v1 = Vertex(v[1].x, v[1].y, v[1].z, t.v1.u, t.v1.v);
            t.v2 = Vertex(v[2].x, v[2].y, v[2].z, t.v2.u, t.v2.v);
        }

        renderTris.push_back(t);
    }

    float const textureWidth = 256;
    float const textureHeight = 256;

    render::begin_frame();
    pvr_list_begin(PVR_LIST_OP_POLY);
    render::draw_textured_quads(renderQuads, static_cast<pvr_ptr_t>(thumbs_up), textureWidth, textureHeight);
    render::draw_textured_tris(renderTris, static_cast<pvr_ptr_t>(thumbs_up), textureWidth, textureHeight);
    pvr_list_finish();
    render::end_frame();
}