/**
 * System includes.
 */

#include <kos.h>
#include <zlib/zlib.h>
#include <cstdlib>
#include <vector>
#include <utility>
#include <arch/timer.h>
#include <iostream>

/**
 * Engine includes.
 */

#include "input.hpp"

/**
 * Game includes.
 */

#include "game/game_jam_game/game_jam_game.hpp"

using std::cout;
using std::endl;

KOS_INIT_FLAGS(INIT_DEFAULT | INIT_MALLOCSTATS);

extern uint8 romdisk[];
KOS_INIT_ROMDISK(romdisk);

uint32_t get_time()
{
    static uint32_t s;
    static uint32_t ms;
    static uint64_t msec;

    timer_ms_gettime(&s, &ms);
    msec = static_cast<uint64_t>(s) * static_cast<uint64_t>(1000) + static_cast<uint64_t>(ms);
    return static_cast<uint32_t>(msec);
}

int main(int argc, char **argv)
{
    srand(get_time() * time(NULL));

    pvr_init_defaults();

    /**
     * Entity.
     */

    GameJamGame * pGameJamGame = new GameJamGame();

    /**
     * Main loop.
     */

    uint32_t last_time = get_time();

    bool done = false;

    while(!done)
    {
        uint32_t dt = 0;

        {
            uint32_t t = get_time();
            dt = t - last_time;
            last_time = t;
        }

        cont_state_t * pFirstControllerState = NULL;

        MAPLE_FOREACH_BEGIN(MAPLE_FUNC_CONTROLLER, cont_state_t, st)

        if(pFirstControllerState == NULL)
        {
            pFirstControllerState = new cont_state_t(*st);
        }

        if(st->buttons & CONT_START && st->buttons & CONT_A)
        {
            done = 1;
            cout << "Exiting!" << endl;
            exit(0);
        }

        MAPLE_FOREACH_END()

        Input input;
        input.pControllerState = pFirstControllerState;
        pGameJamGame->Process(dt, &input);
        delete pFirstControllerState;

        pGameJamGame->Draw();
    }

    delete pGameJamGame;
    return 0;
}
