#include <cstdio>
#include <fstream>
#include <iostream>
#include <cstdlib>

using namespace std;

#include "mesh.hpp"
#include "math_types.hpp"

Mesh::Mesh()
{}

Mesh::~Mesh()
{}

bool Mesh::LoadFromFile(std::string const & sFilePath)
{
    ifstream file(sFilePath, std::ios::in | std::ios::binary);

    if(file)
    {
        uint32_t vertexCount = 0;
        file.read((char*) &vertexCount, sizeof(vertexCount));
        cout << "There are " << vertexCount << " vertices" << endl;

        for(size_t i = 0; i < vertexCount; i += 3)
        {
            Vertex verts[3] = {
                Vertex(), Vertex(), Vertex()
            };

            for(size_t j = 0; j < 3; ++j)
            {
                Vertex & v = verts[j];

                file.read((char*) &v.x, sizeof(v.x));
                file.read((char*) &v.y, sizeof(v.y));
                file.read((char*) &v.z, sizeof(v.z));
                file.read((char*) &v.u, sizeof(v.u));
                file.read((char*) &v.v, sizeof(v.v));
                /**
                file.read((char*) &v.nx, sizeof(v.x));
                file.read((char*) &v.ny, sizeof(v.y));
                file.read((char*) &v.nz, sizeof(v.z));
                **/

                file.read((char*) &v.r, sizeof(v.r));
                file.read((char*) &v.g, sizeof(v.g));
                file.read((char*) &v.b, sizeof(v.b));

                //v.u = 1 - v.u;
                v.v = 1 - v.v;

                //cout << "Vertex " << j << endl;

                //cout << v.x << ", " << v.y << ", " << v.z << " : " << v.u << ", " << v.v << " : " << v.r << ", " << v.g << ", " << v.b << endl;
            }

            triangles.push_back(Tri(verts[0], verts[1], verts[2]));
        }

        //cout << "--- Finished reading, there were " << triangles.size() << " triangles" << endl;

        // Transform mesh data from Blender co-ordinates into our system coordinates.
        // Blender uses a right-handed system with Z pointing up.
        // We use a left-handed system with Z pointing away from the viewer.

        Quaternion q(Vector(1, 0, 0), 90.f * (pi / 180.f));
        Matrix m = q.ToMatrix();

        for(size_t i = 0; i < triangles.size(); ++i)
        {
            Tri & t = triangles[i];

            Vector v[3] = {
                Vector(t.v0.x, t.v0.y, t.v0.z),
                Vector(t.v1.x, t.v1.y, t.v1.z),
                Vector(t.v2.x, t.v2.y, t.v2.z)
            };

            for(size_t j = 0; j < 3; ++j)
            {
                Transform(v[j], m);
            }

            t.v0 = Vertex(v[0].x, v[0].y, v[0].z, t.v0.u, t.v0.v, t.v0.r, t.v0.g, t.v0.b);
            t.v1 = Vertex(v[1].x, v[1].y, v[1].z, t.v1.u, t.v1.v, t.v1.r, t.v1.g, t.v1.b);
            t.v2 = Vertex(v[2].x, v[2].y, v[2].z, t.v2.u, t.v2.v, t.v2.r, t.v2.g, t.v2.b);
        }

        //exit(0);

        cout << "Done" << endl;

        return true;
    }

	return false;
}

tTris & Mesh::GetTris()
{
    return triangles;
}   