#include "render.hpp"
#include <iostream>
#include <cmath>

using std::cout;
using std::endl;

namespace render
{

void draw_textured_tris(tTris & tris, pvr_ptr_t tex, int width, int height)
{
    pvr_poly_cxt_t cxt;
    pvr_poly_hdr_t hdr;
    pvr_vertex_t vert;

    pvr_poly_cxt_txr(&cxt, PVR_LIST_OP_POLY, PVR_TXRFMT_RGB565, width, height, tex, PVR_FILTER_BILINEAR);
    pvr_poly_compile(&hdr, &cxt);
    pvr_prim(&hdr, sizeof(hdr));

    vert.oargb = 0;

    for(size_t i = 0; i < tris.size(); ++i)
    {
        vert.flags = PVR_CMD_VERTEX;

        Tri & t = tris[i];

        vert.argb = PVR_PACK_COLOR(t.v0.a, t.v0.r, t.v0.g, t.v0.b);
        vert.x = t.v0.x;
        vert.y = t.v0.y;
        vert.z = t.v0.z;
        vert.u = t.v0.u;
        vert.v = t.v0.v;

        pvr_prim(&vert, sizeof(vert));

        vert.argb = PVR_PACK_COLOR(t.v1.a, t.v1.r, t.v1.g, t.v1.b);
        vert.x = t.v1.x;
        vert.y = t.v1.y;
        vert.z = t.v1.z;
        vert.u = t.v1.u;
        vert.v = t.v1.v;

        pvr_prim(&vert, sizeof(vert));

        vert.argb = PVR_PACK_COLOR(t.v2.a, t.v2.r, t.v2.g, t.v2.b);
        vert.x = t.v2.x;
        vert.y = t.v2.y;
        vert.z = t.v2.z;
        vert.u = t.v2.u;
        vert.v = t.v2.v;

        vert.flags = PVR_CMD_VERTEX_EOL;
        pvr_prim(&vert, sizeof(vert));
    }
}

void draw_textured_quads(tQuads & quads, pvr_ptr_t tex, int width, int height)
{
    pvr_poly_cxt_t cxt;
    pvr_poly_hdr_t hdr;
    pvr_vertex_t vert;

    pvr_poly_cxt_txr(&cxt, PVR_LIST_OP_POLY, PVR_TXRFMT_RGB565, width, height, tex, PVR_FILTER_BILINEAR);
    pvr_poly_compile(&hdr, &cxt);
    pvr_prim(&hdr, sizeof(hdr));

    vert.oargb = 0;

    for(size_t i = 0; i < quads.size(); ++i)
    {
        vert.flags = PVR_CMD_VERTEX;

        Quad & q = quads[i];

        vert.argb = PVR_PACK_COLOR(q.v0.a, q.v0.r, q.v0.g, q.v0.b);
        vert.x = q.v0.x;
        vert.y = q.v0.y;
        vert.z = q.v0.z;
        vert.u = q.v0.u;
        vert.v = q.v0.v;
        pvr_prim(&vert, sizeof(vert));

        vert.argb = PVR_PACK_COLOR(q.v1.a, q.v1.r, q.v1.g, q.v1.b);
        vert.x = q.v1.x;
        vert.y = q.v1.y;
        vert.z = q.v1.z;
        vert.u = q.v1.u;
        vert.v = q.v1.v;
        pvr_prim(&vert, sizeof(vert));

        vert.argb = PVR_PACK_COLOR(q.v1.a, q.v1.r, q.v1.g, q.v1.b);
        vert.x = q.v2.x;
        vert.y = q.v2.y;
        vert.z = q.v2.z;
        vert.u = q.v2.u;
        vert.v = q.v2.v;
        pvr_prim(&vert, sizeof(vert));

        vert.argb = PVR_PACK_COLOR(q.v1.a, q.v1.r, q.v1.g, q.v1.b);
        vert.x = q.v3.x;
        vert.y = q.v3.y;
        vert.z = q.v3.z;
        vert.u = q.v3.u;
        vert.v = q.v3.v;

        vert.flags = PVR_CMD_VERTEX_EOL;
        
        pvr_prim(&vert, sizeof(vert));
    }
}

void begin_frame()
{
    pvr_wait_ready();
    pvr_scene_begin();
}

void end_frame()
{
    pvr_scene_finish();
}

}