# Required Blender information.

bl_info = {
    "name": "Ross Mesh Exporter",
    "author": "",
    "version": (1, 0),
    "blender": (2, 77, 0),
    "location": "File > Export > RossMesh (.ross_mesh)",
    "description": "Exports mesh data for Ross",
    "warning": "",
    "wiki_url": "",
    "tracker_url": "",
    "category": "Import-Export"
}

# Import required modules.

import array
import bpy
from bpy_extras.io_utils import ExportHelper

class RossMeshExporter(bpy.types.Operator, ExportHelper):
    
    bl_idname = "export_scene.ross_mesh_exporter"
    bl_label = "Export Mesh"
    bl_options = {'PRESET'}
 
    filename_ext = '.ross_mesh'
 
    def execute(self, context):
        
        if (self.filepath == ""):
            print("No sutable filename was provided to save to.")
            return {'FINISHED'}
            
        self.export_active_object()
            
        # Parse all the objects in the scene.
        return {'FINISHED'}
    
    def export_active_object(self):
        
        # copy data from the active object
        # (currently limited to one object at a time)

        obj = bpy.context.active_object
        
        vertex_data = array.array('f')
        
        vertex_count = 0
        uv_idx_base = 0
        
        cols = obj.data.vertex_colors
        
        def idcs_to_verts(idcs):
            result = []
            for idx in idcs:
                result.append(obj.data.vertices[idx])
            return result
    
        for f in obj.data.polygons:
            
            idcs = f.vertices
            verts = []
            uv_idcs = []
            
            if len(idcs) == 4:
                
                verts.extend(idcs_to_verts([idcs[0], idcs[1], idcs[2]]))
                verts.extend(idcs_to_verts([idcs[2], idcs[3], idcs[0]]))
                uv_idcs.extend([0, 1, 2, 2, 3, 0])
                
            elif len(idcs) == 3:
                
                verts.extend(idcs_to_verts(idcs))
                uv_idcs.extend([0, 1, 2])
            
            for i, v in enumerate(verts):
                
                for val in v.co:
                    vertex_data.append(val)
                    
                uv_idx = uv_idx_base + uv_idcs[i]
                    
                uv = obj.data.uv_layers.active.data[uv_idx].uv
                
                print('uv for idx {} is {}'.format(uv_idx, uv))
                
                vertex_data.append(uv.x)
                vertex_data.append(uv.y)
                
                #for val in v.normal:
                #    vertex_data.append(val)
                    
                if len(cols) != 0:
                    c = cols[0].data[uv_idx].color
                    print('Valid vert color! {}'.format(c))
                else:
                    c = (1, 1, 1)
                    
                vertex_data.append(c[0])
                vertex_data.append(c[1])
                vertex_data.append(c[2])
                
            vertex_count += len(verts)
            uv_idx_base += len(idcs)
                
        print('vertex count: ' + str(vertex_count))

        name = obj.name
        with open(self.filepath, 'wb') as f:

            vcount = array.array('I')
            vcount.append(vertex_count)
            vcount.tofile(f)
            
            vertex_data.tofile(f)
            
        with open(self.filepath, 'rb') as f:
            
            import struct
            
            bytes = f.read(4)
            
            print('bytes: ' + str(bytes))
            
            result = struct.unpack('I', bytes)

            print(result)
            
        def register():
            bpy.utils.register_class(ExportMeshPanel)
    
        def unregister():
            bpy.utils.unregister_class(ExportMeshPanel)
 
# Define a function to create the menu option for exporting.
def create_menu(self, context):
   self.layout.operator(RossMeshExporter.bl_idname,text="Ross Mesh (.ross_mesh)")

# Define the Blender required registration functions.
def register():
   """
   Handles the registration of the Blender Addon.
   """
   bpy.utils.register_module(__name__)
   bpy.types.INFO_MT_file_export.append(create_menu)

def unregister():
   """
   Handles the unregistering of this Blender Addon.
   """
   bpy.utils.unregister_module(__name__)
   bpy.types.INFO_MT_file_export.remove(create_menu)

# Handle running the script from Blender's text editor.
if (__name__ == "__main__"):
   print("Registering.")
   register()

   print("Executing.")
   bpy.ops.export_scene.ross_mesh_exporter()